
//TODO: maybe make stop events only fire if no remaining karts in route! 
//TODO: investigate effects on multiplayer, especially SFX-as-music. Maybe disable in mulitplayer.

bool playing_anthem = false;
bool playing_teddy = false;

enum icecream_states
{
  MELTED,
  CARAMEL,
  VANILLA,
  STRAWBERRY,
  NONE
};
int icecream = NONE;
array<string> icecreams = {"icecream_melted",
                           "icecream_caramel",
                           "icecream_vanilla",
                           "icecream_strawberry"};


enum route_states
{
  NORMAL_TRACK,
  TRANSITION,
  DEATH_ROUTE
};

array<int> route_state(Track::getNumberOfKarts(), NORMAL_TRACK);
int local_player_kart = -1;

bool isMultiplayer() {
    return Track::getNumLocalPlayers() > 1;
}

bool isNetworked() {
    return Utils::isNetworking();
}

void onStart()
{
    // terminate any ongoing SFX-as-music and wait until start of race (5 seconds) before starting it again
    // this is a hacky response to SFX not ending after a restart
    Track::getTrackObject("", "sfx_as_music_anthem").getSoundEmitter().stop();
    Track::getTrackObject("", "sfx_as_music_teddy").getSoundEmitter().stop();
    updateIcecreams();
    Utils::setTimeout("playMusic", 5.0);
}

void playMusic()
{
    Track::getTrackObject("", "sfx_as_music_teddy").getSoundEmitter().playLoop();
    playing_teddy = true;
}


// Should be triggered immediately. This function gets the ID of the local player's kart and records it.
// HACK: This takes advantage of unexpected return value of action trigger callback, which returns the camera's kart, not the kart which triggered the function call.
void recordPlayerKart(int idKart) {
    GUI::displayOverlayMessage(formatInt(idKart));
    if (!isMultiplayer()) { // multiplayer is currently unsupported
        local_player_kart = idKart;
    }
}

// Calculates whether the local kart is inside a sphere centered at the given trigger, with given radius
// Assumes local_player_kart > -1
bool isLocalPlayerInTrigger(const string trigger_name, float radius){
    Vec3 loc_kart = Kart::getLocation(local_player_kart);
    Vec3 loc_trigger = Track::getTrackObject("", trigger_name).getCenterPosition();
    if ( 
         (( loc_trigger.getX() - loc_kart.getX() ) * ( loc_trigger.getX() - loc_kart.getX() )) + 
         (( loc_trigger.getY() - loc_kart.getY() ) * ( loc_trigger.getY() - loc_kart.getY() )) +
         (( loc_trigger.getZ() - loc_kart.getZ() ) * ( loc_trigger.getZ() - loc_kart.getZ() ))
         <= (radius * radius) ){
        return true;
    }
    return false;
}

void DecrementIcecream() {
   if (icecream != NONE && icecream > 0) {
      icecream--;
      Utils::setTimeout("DecrementIcecream", 5.0);
      GUI::displayOverlayMessage(formatInt(icecream));
      updateIcecreams();
   }
}

void collectIcecream (int idKart, const string library_instance_id, const string obj_id) {
    if (idKart == local_player_kart) {
        if(icecream == MELTED){GUI::displayOverlayMessage("Melted");}
        if(icecream == CARAMEL){GUI::displayOverlayMessage("Caramel");}
        if(icecream == VANILLA){GUI::displayOverlayMessage("Vanilla");}
        if(icecream == STRAWBERRY){GUI::displayOverlayMessage("Strawberry");}

        Track::getTrackObject("", obj_id).setEnabled(false);

        icecream = NONE;
        updateIcecreams();
    }
}

void updateIcecreams() {
    int i = 0;
    while(i < NONE)
    {
        // if the this object is the current icecream flavour, set enabled. Otherwise set false.
        Track::getTrackObject("", icecreams[i]).setEnabled(icecream == i);
        i++;
    }
}

// run when nearing the death route
void transitionIntoDeathRoute(int idKart)
{
    if (local_player_kart > -1 && route_state[local_player_kart] == NORMAL_TRACK
        && isLocalPlayerInTrigger("road_expert_transition", 20.0)){
        Track::getTrackObject("", "sfx_as_music_teddy").getSoundEmitter().stop();
        playing_teddy = false;
        Track::getTrackObject("", "sfx_death_route_alert").getSoundEmitter().playOnce();
        GUI::displayOverlayMessage("transition");
        route_state[local_player_kart] = TRANSITION;
    }
}

// run when leaving the death route via entrance
void retreatDeathRoute(int idKart)
{
    if (local_player_kart > -1 && route_state[local_player_kart] != NORMAL_TRACK
        && isLocalPlayerInTrigger("road_expert_retreat", 50.0)){
        GUI::displayOverlayMessage("retreat");
        Track::getTrackObject("", "sfx_as_music_anthem").getSoundEmitter().stop();
        playing_anthem = false;
        Track::getTrackObject("", "sfx_as_music_teddy").getSoundEmitter().playLoop();
        playing_teddy = true;
        route_state[local_player_kart] = NORMAL_TRACK;
    }
}

void startDeathRoute(int idKart)
{
    if (local_player_kart > -1 && route_state[local_player_kart] == TRANSITION
        && isLocalPlayerInTrigger("road_expert_start", 5.0)){
        GUI::displayOverlayMessage("start");
        if (!playing_anthem) {
        //TODO make intro and mainloop - 
            Track::getTrackObject("", "sfx_as_music_anthem").getSoundEmitter().playLoop();
            playing_anthem = true;
        }
        icecream = STRAWBERRY;
        updateIcecreams();
        Utils::setTimeout("DecrementIcecream", 20.0 - 10.0);
        route_state[local_player_kart] = DEATH_ROUTE;
    }
}

// run when collecting the ice-cream cone (TODO use trigger to force collection of ice-cream)
void finishDeathRoute(int idKart)
{
    if (local_player_kart > -1 && route_state[local_player_kart] == DEATH_ROUTE
        && isLocalPlayerInTrigger("road_expert_finish", 10.0)){
        GUI::displayOverlayMessage("win");
        Track::getTrackObject("", "sfx_as_music_anthem").getSoundEmitter().stop();
        playing_anthem = false;
        //TODO play 'shloip!' sound
        Track::getTrackObject("", "sfx_death_route_finish").getSoundEmitter().playOnce();
        //play Teddy again after 2 seconds
        Utils::setTimeout("playMusic", 2.0);
        playing_teddy = true;
        //TODO multiply velocity?
        route_state[local_player_kart] = NORMAL_TRACK;
        GUI::displayOverlayMessage("fin");
    }
}

bool test_bool(Track::TrackObject@ obj)
{
    //lazily picking something true
    int unlocked_challenges = Challenges::getCompletedChallengesCount();
    int challenges = Challenges::getChallengeCount();
    return unlocked_challenges >= challenges - 1;
}
void test_cube(int idKart, const string library_instance_id, const string obj_id)
{
    GUI::displayOverlayMessage(":S");
    GUI::displayOverlayMessage(library_instance_id);
    //Track::getTrackObject("", "sfx_as_music_teddy").getSoundEmitter().playLoop();

    GUI::displayOverlayMessage(obj_id);
}

void test_obj (int idKart, const string library_instance_id, const string obj_id)
{
    GUI::displayOverlayMessage(":S");
    GUI::displayOverlayMessage(formatInt(idKart));

    GUI::displayOverlayMessage(obj_id);
}


